import { useEffect, useMemo, useState } from "react";
export const useForm = (initialForm = {}, formValidations = {}) => {
    // State for form values
    const [formState, setFormState] = useState(initialForm);
    // State for form validation messages
    const [formValidation, setFormValidation] = useState({});
    // State for form submission status
    const [isSubmitted, setIsSubmitted] = useState(false);
    // Effect to create validators on component mount
    useEffect(() => {
        createValidators(formState);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    // Memoized value to check if form is valid
    const isFormValid = useMemo(() => {
        if (Object.keys(formState).length === 0)
            return false;
        for (const formValue of Object.keys(formValidation)) {
            if (formValidation[formValue] !== null)
                return false;
        }
        return true;
    }, [formValidation, formState]);
    // Handler for input change events
    const onInputChange = (event) => {
        const target = event.target;
        let value;
        if (target instanceof HTMLInputElement) {
            if (target.type === "checkbox") {
                value = target.checked;
            }
            else if (target.type === "file" && target.files) {
                value = target.files[0];
            }
            else {
                value = target.value;
            }
        }
        else {
            value = target.value;
        }
        const name = target.name;
        // Update form state with new value and re-validate
        setFormState((prevState) => {
            const newState = {
                ...prevState,
                [name]: value,
            };
            createValidators(newState);
            return newState;
        });
    };
    // Handler to reset form to initial state
    const onResetForm = () => {
        setFormState(() => {
            const newState = initialForm;
            createValidators(newState);
            return newState;
        });
    };
    // Function to create validators for form fields
    const createValidators = async (state) => {
        const formCheckedValues = {};
        for (const formField of Object.keys(formValidations)) {
            const [fn, errorMessage] = formValidations[formField];
            const value = state[formField];
            const isValid = await Promise.resolve(fn(value));
            formCheckedValues[`${formField}Valid`] = isValid ? null : errorMessage;
        }
        // Update form validation state
        setFormValidation(formCheckedValues);
    };
    // Handler for form submission
    const handleSubmit = (onSubmit) => {
        return async (event) => {
            event.preventDefault();
            if (isFormValid) {
                setIsSubmitted(true);
                await onSubmit(formState);
                setIsSubmitted(false);
            }
        };
    };
    // Return form state, handlers and validation state
    return {
        ...formState,
        formState,
        setFormState,
        onInputChange,
        onResetForm,
        formValidation,
        ...formValidation,
        isFormValid,
        handleSubmit,
        isSubmitted,
    };
};
//# sourceMappingURL=useForm.js.map